function [discharge,Q] = VectrinoProfiler_discharge(experiment,channel)
% This script is meant to estimate the discharge from a series of flow
% velocity measurements with a Vectrino Profiler. Following input
% parameters should be checked before running:
% --> filenameADV
% --> Nm
% --> Np
% --> summertime
% --> dtPerPoint
% --> date_start
% --> 3 correction parameters for Y,Z en WL
% --> cellsize and distance between probe and centre of first measurement cell
%
% NOTE: the bed level reading from the Vectrino Profiler is in the
% calculations not corrected to the conventions of a zero-reading at the
% flume bottom.

% INPUT PARAMETERS:
% experiment (e.g. 'A1' or 'C2')
% channel = 'side_mid'; %'main_up', 'main_down', 'side_down', 'main_mid', 'side_mid'

year = 2018;

correctionY = 72; % y = yADV + correctionY
correctionZ = -751; % z = -(zADV + z0ADV + correctionZ)
correctionWL = 58.4; % waterlevel = WL + correctionWL

cellsize = 1; %mm
cellstart = 40.5; %mm

if strcmp(channel(1:4),'main')
    dY = 100; % cell size in y-direction (in mm)
elseif strcmp(channel(1:4),'side')
    dY = 50;
else
    error('Channel name is not correct.');
end

%%
if strcmp(experiment(end),'1') && strcmp(channel(end-2:end),'mid')
    error('No velocity measurements available at x=8500mm for the low water experiments.');
end

%%
filenameWL = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/Exp%s/waterlevel/';
filenameADV = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/Exp%s/Vectrino/discharge/VectrinoData.%03d.18.Vectrino Profiler.%05d_%d.mat';
filenamePath = '~/Documents/abstracts_papers/PAPER2_lab/experiments_LTD/Exp%s/Vectrino/discharge/';

switch experiment
    case 'A1'
        filenameWL = [filenameWL 'lin_sens_20180220_flowing.txt'];
        Nn = 53; summertime = 0;
        filenamePath = [filenamePath 'Vectrino_discharge.dat'];
    case 'A2'
        filenameWL = [filenameWL 'lin_sens_20180227.txt'];
        Nn = 59; summertime = 0;
        filenamePath = [filenamePath 'A2_Daan_debiet_28022018.dat'];
    case 'AB1'
        filenameWL = [filenameWL 'lin_sens_20180306_flowing.txt'];
        Nn = 67; summertime = 0;
        filenamePath = [filenamePath 'Discharge_Daan_08030218.dat'];
    case 'AB2'
        filenameWL = [filenameWL 'lin_sens_20180314.txt'];
        Nn = 73; summertime = 0;
        filenamePath = [filenamePath 'DIS_DAAN_herstart_14032018_adapted.dat'];
    case 'B1'
        filenameWL = [filenameWL 'lin_sens_20180320_flowing.txt'];
        Nn = 82; summertime = 0;
        filenamePath = [filenamePath 'DIS_daan_23032018.dat'];
    case 'B2'
        filenameWL = [filenameWL 'lin_sens_20180326_flowing.txt'];
        Nn = 87; summertime = 1;
        filenamePath = [filenamePath 'DIS_DAAN_28032018.dat'];
    case 'C1'
        filenameWL = [filenameWL 'lin_sens_20180604_still.txt'];
        Nn = 158; summertime = 1;
        filenamePath = [filenamePath 'Discharge_07_06_2018.dat'];
    case 'C2'
        filenameWL = [filenameWL 'lin_sens_20180612_RESTART.txt'];
        Nn = 164; summertime = 1;
        filenamePath = [filenamePath 'DIS_13_06_2018.dat'];
    case 'D1'
        filenameWL = [filenameWL 'lin_sens_20180621_flowing.txt'];
        Nn = 173; summertime = 1;
        filenamePath = [filenamePath 'Vectrino_grid_discharge.dat'];
    case 'D2'
        filenameWL = [filenameWL 'lin_sens_20180625_flowing_v2.txt'];
        Nn = 178; summertime = 1;
        filenamePath = [filenamePath 'Vectrino_grid_discharge.dat'];
end

if strcmp(experiment,'AB2')
    Nm=1;
else
    Nm=0;
end

Np = numel(strfind(ls(sprintf(strrep(filenameADV,'%d','*'),experiment,Nn,Nm)),'.mat')); % number specifying number of sub-files of one large Vectrino measurement

date_start = datenum(year,1,Nn);

%%
%%%
% Import the corresponding water level file of the linear
% sensors in column vectors time_WL and WL
%%%
WL_data = importdata(sprintf(filenameWL,experiment),' ',4);

Q = WL_data.data(:,2) + WL_data.data(:,11);

switch channel
    case 'main_up'
        WL = WL_data.data(:,15); % WL124
    case 'main_down'
        WL = WL_data.data(:,9); % WL113
    case 'side_down'
        WL = WL_data.data(:,7); % WL111
    case 'main_mid'
        WL = WL_data.data(:,10); % WL114
    case 'side_mid'
        WL = WL_data.data(:,8); % WL112
end
waterlevel = WL + correctionWL;
time_WL = datetime(strcat(WL_data.textdata(5:end,1),{' '},WL_data.textdata(5:end,2)),'InputFormat','dd-MM-yyyy HH:mm:ss');
time_WL_num = datenum(time_WL);

[timeStarttot,Xpostot,Ypostot,z0ADVtot,DLYtot,timeEndtot] = import_location_discharge(sprintf(filenamePath,experiment));

switch channel
    case 'main_up'
        dum_ind = Ypostot<1600 & Xpostot==2000;
    case 'main_down'
        dum_ind = Ypostot<1600 & Xpostot==12000;
    case 'side_down'
        dum_ind = Ypostot>1600 & Xpostot==12000;
    case 'main_mid'
        dum_ind = Ypostot<1600 & Xpostot==8500;
    case 'side_mid'
        dum_ind = Ypostot>1600 & Xpostot==8500;
end

DLY = DLYtot(dum_ind);
timeEnd = timeEndtot(dum_ind);
z0ADV = z0ADVtot(dum_ind);
timeStart = timeStarttot(dum_ind);
Ypos = Ypostot(dum_ind);

%%

Ypos2 = Ypos + correctionY;
Zpos2 = -z0ADV -correctionZ;

U = [];
Cor = [];
T = [];
bed = [];
Tb = [];

%%

for i=1:Np
    load(sprintf(filenameADV,experiment,Nn,Nm,i));
    U = [U; -Data.Profiles_VelY];
    Cor = cat(1,Cor,cat(3,Data.Profiles_CorBeam1,Data.Profiles_CorBeam2,Data.Profiles_CorBeam3,Data.Profiles_CorBeam4));
    T = [T; Data.Profiles_HostTimeMatlab(:)];
    bed = [bed; Data.BottomCheck_BottomDistance(:)];
    Tb = [Tb; Data.BottomCheck_HostTimeMatlab(:)];
end

%%
Zpos2(~strcmp(DLY,'DLY10')) = [];

date = today;
timeEndNum = datenum(timeEnd)-(date-date_start)-(1+summertime)/24;
timeStartNum = datenum(timeStart)-(date-date_start)-(1+summertime)/24;

for i=1:length(timeEndNum)-1
    if timeEndNum(i+1)<timeEndNum(i)
        timeEndNum(i+1) = timeEndNum(i+1)+1;
    end
    if timeStartNum(i+1)<timeStartNum(i)
        timeStartNum(i+1) = timeStartNum(i+1)+1;
    end
end

%%

dumWL1 = find(time_WL_num >= timeStartNum(1) & time_WL_num <= timeEndNum(end),1);
dumWL2 = find(time_WL_num >= timeStartNum(1) & time_WL_num <= timeEndNum(end),1,'last');
if ~isempty(dumWL1) && ~isempty(dumWL2)
    waterlevel = mean(waterlevel(dumWL1:dumWL2));
    Q = mean(Q(dumWL1:dumWL2));
elseif ~isempty(dumWL1) && isempty(dumWL2)
    waterlevel = mean(waterlevel(dumWL1:end));
    Q = mean(Q(dumWL1:end));
    disp([experiment ': The waterlevel is calculated over a shorter period.'])
else
    waterlevel = mean(waterlevel(end-59):waterlevel(end));
    Q = mean(Q(end-59):Q(end));
    disp([experiment ': The waterlevel is calculated using the last minute of the file, because the exact timeslot was not available.'])
end

upper_size = waterlevel - (Zpos2 - cellstart + cellsize/2);

%%
Npoints = length(Ypos2);
U2 = cell(Npoints,1);
Cor2 = U2;
T2 = U2;
Tb2 = U2;
bed2 = U2;

for i=1:Npoints
    indCell = find(T>=timeStartNum(i) & T<=timeEndNum(i));
    T2{i} = T(indCell);
    U2{i} = U(indCell,:);
    Cor2{i} = Cor(indCell,:,:);
    
    indCellBED = find(Tb>=timeStartNum(i) & Tb<=timeEndNum(i));
    Tb2{i} = Tb(indCellBED);
    bed2{i} = bed(indCellBED);
end

%%

ind_relevant = find(strcmp(DLY,'DLY10'));
Ypos2 = Ypos2(ind_relevant);
U2 = U2(ind_relevant);
Cor2 = Cor2(ind_relevant);
T2 = T2(ind_relevant);

if max(Ypos2) == Ypos2(1)
    Ypos2 = flipud(Ypos2);
elseif min(Ypos2) ~= Ypos2(1)
    prompt = input('The order of measured Y-values is different than expected. This may lead to unexpected outcomes. Do you wish to proceed? Y/N [N] ','s');
    if ~strcmp(prompt,'Y') && ~strcmp(prompt,'y') && ~strcmp(prompt,'yes') && ~strcmp(prompt,'Yes')
        error('Script terminated by user.');
    end
end

%%
[U2_filt2,~,~,~] = ADV_Preprocessing(U2,[],[],[],T2,Cor2);

%%
bed2 = bed2(ind_relevant);
Tb2 = Tb2(ind_relevant);

%%
%%% remove bed level data differing more than 1 standard deviation from one
%%% of neighbours
Tb2_filt2 = cell(size(Tb2));
bed2_filt2 = Tb2_filt2;

for i=1:length(bed2)
    ind_bed = ~logical(max([0; abs(diff(bed2{i})>nanstd(bed2{i}))],[abs(diff(bed2{i})>nanstd(bed2{i})); 0]));
    Tb2_filt2{i} = Tb2{i}(ind_bed);
    bed2_filt2{i} = bed2{i}(ind_bed);
end

%%
%%%%%%%%%%
discharge = 0;
for i=1:length(Ypos2)
    Umean = nanmean(U2_filt2{i});
    bedmean = nanmean(bed2_filt2{i});
    for j=1:length(Umean)
        if ~isnan(Umean(j))
            discharge = discharge + Umean(j)*dY/1000*cellsize/1000; % integral over measured velocity profile
        end
    end
    discharge = discharge + Umean(find(arrayfun(@(x)~isnan(x),Umean)==1,1))*dY/1000*upper_size(i)/1000; % add top velocity integrated towards water surface
    
    if bedmean>(cellstart+cellsize*29)/1000
        discharge = discharge + 0.5*(bedmean-(cellstart+cellsize*29.5)/1000)*Umean(find(arrayfun(@(x)~isnan(x),Umean)==1,1,'last'))*dY/1000; % add bottom velocity integrated towards bed
    end
    
    slope = -min(nanmean(diff(Umean(1:7))),0)/(cellsize/1000);
    discharge = discharge + 0.5*slope*Umean(find(arrayfun(@(x)~isnan(x),Umean)==1,1))*upper_size(i)/1000*dY/1000; % add linear interpolated correction to top layer integration
end

discharge = discharge * 1000;

