function printfigure(name,varargin)
% PRINTFIGURE  prints figure in different formats.
%    printfigure(NAME) prints current figure in eps, pdf, png, and jpg 
%    formats with filename NAME. The optional HANDLE input can be used
%    if figure is not current. 
%
%    printfigure(...,TYPE) TYPE equals 'vector' (the default) or 'raster'
%    and sets the type of output for the eps and pdf files
%
%    printfigure(...,COLOR) COLOR equals 'rgb' or 'cmyk' (the default)
%    which sets the color model to be used (only for postscipt images)
%
%    printfigure(...,hf) handle of the figure to print. hf should be a
%    valid handle graphics handle
%
%    See also: makefigure
%
%    Author: Ryan Teuling and Bart Vermeulen
%
%    Last edit: 07-08-2013

P=inputParser;
P.FunctionName='printfigure';
P.addRequired('name',@(x) ischar(x) && isvector(x))
P.addOptional('type','vector',@(x) ischar(x) && isrow(x) && any(strcmpi(x,{'vector','raster'})))
P.addOptional('color','cmyk',@(x) ischar(x) && isrow(x) && any(strcmpi(x,{'cmyk','rgb'})))
P.addOptional('hf',gcf,@(x) ishghandle(x) && isequal(get(x,'type'),'figure'))
P.parse(name,varargin{:});
hf=P.Results.hf;
if strcmpi(P.Results.color,'cmyk'), cmode='-cmyk'; else cmode=[]; end
type=P.Results.type;

if ispc || ismac
%   print(hf,'-dpng','-zbuffer',cmode,'-r600',[name '.png']);
    print(hf,'-djpeg90','-zbuffer',cmode,'-r600',[name '.jpg']);
    switch type       
        case 'vector'
            print(hf,'-dpdf','-painters',cmode,'-loose',[name '.pdf']);
            print(hf,'-depsc2','-painters',cmode,'-loose',[name '.eps']); 
        case 'raster'
            print(hf,'-depsc2','-zbuffer',cmode,'-r600','-loose',[name '.eps']);
            print(hf,'-dpdf','-zbuffer',cmode,'-r600','-loose',[name '.pdf']);
    end
else
    switch type
        case 'vector'
            try
                plot2svg([name '.svg'],hf); % export svg file
                system(['inkscape ',name,'.svg --export-pdf=',name,'_from_svg.pdf']); % use inkscape to convert svg to pdf
            catch
                warning('Trouble generating svg file!')
            end
            print(hf,'-dpdf','-painters',cmode,'-loose',[name '.pdf']);
        case 'raster'
            if ~strcmpi(cmode,'-cmyk')
                print(hf,'-dtiff','-opengl','-r600',[name,'.tiff'])
            end
            print(hf,'-dpdf','-opengl',cmode,'-r600','-loose',[name '.pdf']);
    end        
    system(['pdftocairo -png -r 600 -singlefile ',name, '.pdf ',name]); 
    system(['pdftocairo -jpeg -r 150 -singlefile ',name, '.pdf ',name]);
    if strcmpi(P.Results.color,'cmyk')
        system(['pdftops -eps -level2sep ',name, '.pdf']);
    else
        system(['pdftops -eps ',name, '.pdf ',name,'.eps']);
    end
end


